@@ -17,7 +17,7 @@ class Image(TypedDict, total=False):
     ImageId: str
     CreationDate: str
     RootDeviceName: Optional[str]
-    Size: int
+    Size: Optional[int]
     # optional
     SnapshotId: str
 
@@ -98,7 +98,7 @@ def describe(
             "ImageId": i["ImageId"],
             "CreationDate": i["CreationDate"],
             "RootDeviceName": i["RootDeviceName"] if "RootDeviceName" in i else None,
-            "Size": i["BlockDeviceMappings"][0]["Ebs"]["VolumeSize"],
+            "Size": i["BlockDeviceMappings"][0]["Ebs"]["VolumeSize"] if i["BlockDeviceMappings"] else None,
         }
         if show_snapshot_id:
             image["SnapshotId"] = i["BlockDeviceMappings"][0]["Ebs"]["SnapshotId"]
