@@ -43,10 +43,11 @@ ec2_cli = [
     Cmd(ec2.launch, [
         config_arg,
         Arg("name", type=str, help="Name tag of instance"),
-        Arg("ami", type=ami_arg_checker, help=f"AMI id or a keyword to lookup the latest ami: {[k for k in ami.ami_keywords.keys()]}"),
+        Arg("-a", "--ami", type=ami_arg_checker, help=f"AMI id or a keyword to lookup the latest ami: {[k for k in ami.ami_keywords.keys()]}"),
+        Arg("-t", "--template", type=str, help="Launch template name"),
         Arg("--volume-size", type=int, help="EBS volume size (GB). Defaults to AMI volume size."),
         Arg("--encrypted", type=bool, help="Whether the EBS volume is encrypted", default=True),
-        Arg("--instance-type", type=str, help="Instance type", default="t2.medium"),
+        Arg("--instance-type", type=str, help="Instance type"),
         Arg("--key-name", type=str, help="Key name"),
         Arg("--userdata", type=str, help="Path to user data file")
     ]),
@@ -69,9 +70,14 @@ ec2_cli = [
     ]),
     Cmd(ec2.tags, [
         config_arg,
+        Arg("name", type=str, nargs='?', help="Filter to instances with this Name tag or instance id."),
+        Arg("-q", type=str, dest='name_match', help="Filter to instances with a Name tag containing NAME_MATCH."),
         Arg("-v", "--volumes", action='store_true', help="Show volumes"),
         Arg("-k", "--keys", type=str, nargs='+', metavar="KEY", help="Tags to display", default = [])
     ]),
+    Cmd(ec2.templates, [
+        config_arg
+    ]),
     Cmd(ec2.terminate, [
         config_arg,
         Arg("name", type=str, help="Name tag of instance or instance id")
